/*
 * Copyright (c)  1998 - 2000, Amnon Barak, The Hebrew University of Jerusalem.
 * All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted
 * under the terms of version 2 or any later version of the GNU General
 * Public License, as published by the Free Software Foundation.
 *
 * THE HEBREW UNIVERSITY OF JERUSALEM  PROVIDES THIS SOFTWARE IN ITS "AS IS"
 * CONDITION. THE HEBREW UNIVERSITY OF JERUSALEM DISCLAIMS  ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 */
/*
 * Author(s): Amnon Shiloh.
 */
#ifndef _MOS_DFSA_H
#define _MOS_DFSA_H

#include <linux/fs.h>
#include <linux/file.h>

#ifdef CONFIG_MOSIX_DFSA

#define	MAXDFSAFS	8

struct dfsatab
{
	unsigned short len;
	char ndefs;
	struct dfsdef
	{
		char defno;
		char mounted;
		short offset;
	} dfsdef[0];
};

/* update flags (dupdates): */

#define	DFSA_UPDCRED	1	/* need to update credentials */
#define	DFSA_UPDTABLE	2	/* need to update DFSA table */
#define	DFSA_UPDCDIR	4	/* need to update current-directory */
#define	DFSA_UPDMAX	8	/* need to update "max_fdset" */
#define	DFSA_UPDCLOSE	16	/* need to update closed files */
#define	DFSA_UPDMODS	32	/* need to update modified files */
#define	DFSA_UPDUSED	64	/* need to update the "used" bits */
#define	DFSA_UPDSEL	128	/* need to update MFS node-selections */

#define	DFSA_UPDATE	(DFSA_UPDCRED|DFSA_UPDTABLE|DFSA_UPDCDIR|DFSA_UPDMAX|\
			 DFSA_UPDCLOSE|DFSA_UPDMODS|DFSA_UPDUSED|DFSA_UPDSEL)

extern struct task_struct *child_reaper;

#define CAN_DFSA(p)	((p)->fs && atomic_read(&(p)->fs->count) == 1 && \
			(p)->files && atomic_read(&(p)->files->count) == 1 && \
			(p)->fs->root == child_reaper->fs->root && \
			!((p)->flags & (PF_PTRACED|PF_TRACESYS|PF_EXITING)))

void dfsa_init(void);
int dfsa_define(int, char *);
char *dfsa_getdir(int, int *);
void dfsa_adapt(struct dfsatab *);
void dfsa_name_check(char *, int);
void dfsa_mount_check(char *, int);
void dfsa_close_file(int);
void dfsa_touch_file(int, int);
int can_dfsa_file(int, int);
int dfsa_sync(int);
void clear_dfsasync(void);
void remote_clear_dfsa(void);
void dfsa_tinit(struct task_struct *);
void deputy_resync_dfsa(void);
char *deputy_pack_dfsa_changes(int *);
void remote_unpack_dfsa_changes(char *);
char *remote_pack_dfsa_changes(int *);
void deputy_unpack_dfsa_changes(char *);
void dfsa_check_comm_send(int *, char **, int *);
void dfsa_comm_recv(int, char *);
int enter_remote_dfsa_mode(void);
void leave_remote_dfsa_mode(void);
void dfsa_syscall_on_file(int, int);
void dfsa_exit(void);
struct dentry *name_starts_in_dfsa(char **);
int is_dfsa_dentry(struct dentry *);
int dfsa_optimized_read(struct file *, loff_t, unsigned long, unsigned long);

struct dentry *dfsa_lookup_wrapper(const char *, struct dentry *, unsigned int);
struct dentry *dfsa_namei_wrapper(const char *, unsigned int);

/* flags for dfsa_touch_file */
#define	DTF_NEW		1	/* must be "1" - since used in the "fd_install"
				 * macro by files that do not include this file
				 */
#define	DTF_FORCE	2	/* touch even if not DFSA'able */
#define	DTF_COUNT	4	/* extra f_count within 'fget-fput' */

#endif CONFIG_MOSIX_DFSA
#endif
