/*
 * Copyright (c)  1998 - 2000, Amnon Barak, The Hebrew University of Jerusalem.
 * All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted
 * under the terms of version 2 or any later version of the GNU General
 * Public License, as published by the Free Software Foundation.
 *
 * THE HEBREW UNIVERSITY OF JERUSALEM  PROVIDES THIS SOFTWARE IN ITS "AS IS"
 * CONDITION. THE HEBREW UNIVERSITY OF JERUSALEM DISCLAIMS  ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 */
/*
 * Author(s): Amnon Shiloh.
 */

/* THIS IS AN OPERATING-SYSTEM INDEPENDENT FILE:		*/
/* DO NOT MENTION HERE ANY STRUCTURES, VARIABLES OR DEFINITIONS	*/
/* FROM THE PARTICULAR HOSTING OPERATING-SYSTEM KERNEL		*/

#ifndef _LINUX_MOSIXTASK_H
#define _LINUX_MOSIXTASK_H

typedef long long m64;
typedef int m32;
typedef unsigned int mu32;
typedef long long now_t;

#include <mos/defs.h>
#include <mos/comm.h>
#include <mos/debug.h>

#ifdef CONFIG_MOSIX_DFSA
struct dfsatab;
#endif CONFIG_MOSIX_DFSA

#ifdef CONFIG_MOSIX_FS
struct dentry;
typedef struct dentry *(*lookup_dentry_t)(const char *, struct dentry *, unsigned int);
struct mfs_stats;
#endif CONFIG_MOSIX_FS

struct opcost
{
	int page, syscall, out, outkb, in, inkb
#ifdef CONFIG_MOSIX_TOPOLOGY
	, first, last
#endif CONFIG_MOSIX_TOPOLOGY
	;
};

struct mosix_task
{
	volatile mu32 dflags;	/* distributed (MOSIX) flags */
	/* following chars/bits should be in dflags, but modified within
	 * interrupt code... if only the compiler generated single-instruction
	 * codes for "|=" and "&=~" (but it doesn't) */
	volatile char senturgent;	/* REMOTE has something urgent */
	volatile char commpri;		/* prioririty for MOSIX communication */
	volatile char ignoreoldsigs;	/* deliberately cleared ->sigpending */
	volatile char catchme;		/* catch for debugging */

	volatile int whereto;		/* migration request */
	mu32 stay;			/* reasons why process must stay */
	struct prequest *prequest;	/* requests to this task */
	int whereami;			/* where the process is */
	int deppe;			/* where the process came from */
	volatile char bstate;		/* backed-up state while in MOSIX */
	struct held_files
	{
		void *f;		/* a file used remotely */
		char denywrite;		/* whether holding i_writecount down */
	} *held_files;			/* files held by remote VM */
	int held_allocated;		/* # of entries in "held_inodes" */
	struct mosix_link *contact;	/* DEPUTY <==> REMOTE connection */
	struct mosix_task *ancesstor;	/* nearest ancesstor when dependent */
	mu32 deputytime;		/* ticks spent on DEPUTY */
	long last_sigxcpu;		/* last [prof] time SIGXCPU was sent */
	m64 passedtime;			/* time already passed to DEPUTY */
	m64 uttime;		/* time to add(REMOTE)/subtract(LOCAL) to/from
				 * times->tms_utime for statistic purposes */
	m64 dctime;		/* decayed user-time (ms) */
	m64 cutime;		/* cummulative dependent-child user-time (ms) */
	m64 pagetime;		/* time waiting for free pages (ms) */
	m64 ndemandpages;	/* statistical # of demand pages */
	m64 nsyscalls;		/* statistical # of of syscalls */
	m64 ncopyouts;		/* statistical # of copying kernel to user */
	m64 copyoutbytes;	/* statsitical # of bytes to user */
	m64 ncopyins;		/* statistical # of copying user to kernel */
	m64 copyinbytes;	/* statistical # of bytes from user */
	mu32 iocounter;		/* combintaion of above statistics */
	struct opcost depcost[MAX_MOSIX_TOPOLOGY]; /* costs on DEPUTY */
	int depspeed;		/* speed of DEPUTY */
	m64 last_consider;	/* user time since last considered migration */
	now_t last_mconsider;	/* when last selected for memory-balancing */
	unsigned short decsecs;	/* seconds within decay cycle */
	unsigned short deccycle;/* length of decay cycle in seconds */
	short decay;		/* decay factor (out of DECAY_QUOTIENT) */
	unsigned char dpolicy;	/* statistic-collection policy */
	mu32 asig;		/* signals arriving on REMOTE */
	int pages_i_bring;	/* # if pages still to be brought */
	int rpagecredit;	/* # of expected remote page-faults */
	int page_allocs;	/* page allocation requests decaying counter */
	short ran;		/* ticks running during last second */
	unsigned short runstart;/* "load_ticks" (+1) since running */
	int nmigs;		/* number of [successful] migrations - ever */
	int load;		/* estimated contribution to load */
	int refs;		/* references to this task - do not destroy */
	int sonpid;		/* pid of son in remote-fork */
	int loadhere;		/* original local load */
	int migpages;		/* # of migrating pages */
	void *inexec;		/* structure while in "exec" */
	void *wait_dist;	/* waiting for change in dflags */
	unsigned long exit_mem;	/* memory on REMOTE when exited */
	unsigned short deputy_regs; /* bit map of regs in charge of DEPUTY */
	unsigned short pass_regs; /* bit map of regs to pass */
	mu32 features;		/* CPU features on original node */
	short mypid;		/* original PID */
	short sigmig;		/* signal to receive on migration */
	mu32 *altregs;		/* place of registers when not on stack-top */
	void *mosix_log;
	struct data_cache *dcache;
	char *ps;		/* common "ps" information */
	int memused, memunused, memswapped;	/* memory split-up */
	unsigned long last_memsort;	/* last time of memory was sorted */
	unsigned long task_size;/* TASK_SIZE on DEPUTY */
#ifdef CONFIG_MOSIX_DFSA
	int copy_ins;
	int bytes_in;
	mu32 dupdates;
	struct dfsatab *ttab;
	char namei_all_in;
#endif CONFIG_MOSIX_DFSA
#ifdef CONFIG_MOSIX_FS
	unsigned short express_flags;
	unsigned short express_function;
	lookup_dentry_t altlookup;
	int iget_mfs_pe;
	unsigned long long iget_mfs_handle;
	long mfs_arg1;
	long mfs_arg2;
	void *mfs_parg;
	int selected;
	int lastexec;
	int lastmagic;
	struct mfs_stats *mfs_stats;
#endif CONFIG_MOSIX_FS
	struct mosix_task *next;
};

#ifdef CONFIG_MOSIX_DFSA
#define	DFSA_INIT_TASK	0, 0, 0, NULL, 0,
#else
#define DFSA_INIT_TASK
#endif CONFIG_MOSIX_DFSA

#ifdef CONFIG_MOSIX_FS
#define	MFS_INIT_TASK	0, 0, NULL, 0, 0, 0, 0, NULL, 0, 0, 0, NULL,
#else
#define	MFS_INIT_TASK
#endif CONFIG_MOSIX_FS
#define MOSIX_INIT_TASK { \
/* dflags */	0,	\
/* senturgent */ 0,	\
/* commpri */	0,	\
/* ignoreoldsigs */ 0,	\
/* catchme */ 0,	\
/* whereto */	0,	\
/* stay */	0,	\
/* prequest */	NULL,	\
/* whereami */	0,	\
/* deppe */	0,	\
/* bstate */	TASK_SAME, \
/* held_files */ 0,	\
/* held_allocated */ 0,	\
/* dsock */	NULL,	\
/* ancesstor */	NULL,	\
/* deputytime */ 0,	\
/* last_sigxcpu */ 0,	\
/* passedtime */ 0,	\
/* uttime */	0,	\
/* dctime */	0,	\
/* cutime */	0,	\
/* pagetime */	0,	\
/* ndemandpages */ 0,	\
/* nsyscalls */ 0,	\
/* ncopyouts */ 0,	\
/* copyoutbytes */ 0,	\
/* ncopyins */ 0,	\
/* copyinbytes */ 0,	\
/* iocounter */ 0,	\
/* depcost */	{{0,0,0,0,0,0}}, \
/* depspeed */	0,	\
/* last_consider */ 0,	\
/* last_mconsider */ 0,	\
/* decsecs */	0,	\
/* deccycle */  0,	\
/* decay */	0,	\
/* dpolicy */	0,	\
/* asig */	0,	\
/* pages_i_bring */ 0,	\
/* rpagecredit */ 0,	\
/* page_allocs */ 0,	\
/* ran */	0,	\
/* runstart */	0,	\
/* nmigs */	0,	\
/* load */	0,	\
/* refs */	0,	\
/* sonpid */	0,	\
/* loadhere */	0,	\
/* migpages */	0,	\
/* inexec */	NULL,	\
/* dist wait */	NULL,	\
/* exit_mem */	0,	\
/* deputy_regs */ 0,	\
/* pass_regs */ 0,	\
/* features */ 0,	\
/* mypid */ 0,		\
/* sigmig */ 0,		\
/* altregs */	NULL,	\
/* mosix_log */ NULL,	\
/* dcache */	NULL,	\
/* ps */	NULL,	\
/* memusage */	0,0,0,	\
/* last_memsort */ 0,   \
/* task_size */ TASK_SIZE, \
DFSA_INIT_TASK \
MFS_INIT_TASK \
/* next */	NULL,	\
}

/*
 * distributed flags (dflags)
 */
#define DDEPUTY		0x00000001	/* process is a DEPUTY stub */
#define DREMOTE		0x00000002	/* process is running remotely */
#define	DINSCHED	0x00000004	/* MOSIX task within "schedule" */
#define DSYNC		0x00000008	/* remote process is synced and waits */
#define DPSYNC		DSYNC		/* deputy must tell us to continue */
#define DNESTED		0x00000010	/* nested requests from remote */
#define DNICECNG	0x00000020	/* priority changed */
#define DHOMEREQUEST	0x00000040	/* someone waits for us to go home */
#define DINCOMING	0x00000080	/* process coming here */
#define DPASSING	0x00000100	/* process is in migration */
#define DPAGEIN		0x00000200	/* in pagein, considered running */
#define DBALANCING	0x00000400	/* selected for load-balancing */
#define DMBALANCING	0x00000800	/* selected for memory-balancing */
#define DFINISHED	0x00001000	/* wants to become zombie */
#define	DUPDOVERHEADS	0x00002000	/* must update overheads */
#define DCHECK_CONF	0x00004000	/* check process location still OK */
#define	DREMOTEDAEMON	0x00008000	/* set DREMOTE on "fork" */
#define	DFAKESIGNAL	0x00020000	/* produce a fake signal */
#define	DNOINTERRUPT	0x00040000	/* prevent signals/events */
#define	DTRACESYS1	0x00080000	/* PF_TRACESYS done before syscall */
#define	DTRACESYS2	0x00100000	/* syscall done before 2nd PF_TRACESYS*/
#define	DREMOTESYS	0x00400000	/* within a remote system-call */
#define	DMOSCAMEUP	0x00800000	/* notice that MOSIX is now up */
#define	DMOSCOMESDOWN	0x01000000	/* notice that MOSIX is coming down */
#define	DMUSTBEBACK	0x02000000	/* MUST arrive back home */
#ifdef CONFIG_MOSIX_DFSA
#define	DSTATSDOWN	0x04000000	/* turn off local system-call stats */
#define	DREMOTEDFSA	0x08000000	/* on REMOTE: within a DFSA syscall */
#define	DSYNCDFSASYS	0x10000000	/* request to sync DFSA information */
#define	DSYNCDFSAOWN	0x20000000	/* process needs to sync DFSA info */
#define	DSYNCDFSA	(DSYNCDFSASYS|DSYNCDFSAOWN)
#endif CONFIG_MOSIX_DFSA

/*
 * reasons to stay:
 */

#define DSTAY_FOR_MONKEY 0x00000001	/* using monkey vnode */
#define DSTAY_FOR_SHM	 0x00000002	/* SYSV shared memory */
#define DSTAY_FOR_DEV	 0x00000004	/* mapping a device */
#define DSTAY_FOR_86	 0x00000008	/* running in 86 mode */
#define DSTAY_ITS_DAEMON 0x00000010	/* daemon process */
#define DSTAY_FOR_PRIV	 0x00000020	/* privilleged inst. access (in/out) */
#define DSTAY_FOR_MLOCK	 0x00000040	/* has locked memory */
#define DSTAY_FOR_CLONE	 0x00000080	/* shared VM */
#define DSTAY_FOR_RT	 0x00000100	/* Real-Time scheduling */
#define	DSTAY_PROC_MAPPED 0x00000200	/* someone maps us via "proc" fs */
#define	DSTAY_FOR_IOPL	0x00000400	/* direct I/O permission */
#define	DSTAY_MAP_PROC	0x00000800	/* mapping a process via "proc" fs */
#define	DSTAY_ITS_INIT	0x00001000	/* init process */
#define	DSTAY_OTHER1	0x01000000	/* external reason for stay (1) */
#define	DSTAY_OTHER2	0x02000000	/* external reason for stay (2) */
#define	DSTAY_OTHER3	0x04000000	/* external reason for stay (3) */
#define	DSTAY_OTHER4	0x08000000	/* external reason for stay (4) */
#define DNOMIGRATE	0x80000000	/* user requested no auto-migrations */

#define DSTAY   (~DNOMIGRATE)

/*
 * where to go (whereto)
 */
#define GOBACKHOME	(-1)	/* just go back home */
#define BALANCE		(-2)	/* perform load balancing */
#define IOBALANCE	(-3)	/* perform load balancing for I/O */
#define MEMBALANCE	(-4)	/* perform balancing for memory */
#define	MFSBALANCE	(-5)	/* perform balancing for MFS */
#define	MUSTGOHOME	(-6)	/* must go back home */

extern void run_on_m(struct mosix_task *);
extern void run_off_m(struct mosix_task *);

#ifdef CONFIG_MOSIX_FS
/* express flags: */
#define	MFS_EXPRESS		1
#define	MFS_EXCL		2		/* O_EXCL on create */
#define	MFS_EXPRESS_DONE	4

/* express functions: */
enum
{
	MFS_EXPRESS_CREATE,
};

#endif CONFIG_MOSIX_FS

#endif
