/*
 * Copyright (c)  1998 - 2000, Amnon Barak, The Hebrew University of Jerusalem.
 * All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted
 * under the terms of version 2 or any later version of the GNU General
 * Public License, as published by the Free Software Foundation.
 *
 * THE HEBREW UNIVERSITY OF JERUSALEM  PROVIDES THIS SOFTWARE IN ITS "AS IS"
 * CONDITION. THE HEBREW UNIVERSITY OF JERUSALEM DISCLAIMS  ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 */
/*
 * Author(s): Amnon Shiloh.
 */
#ifndef _MOS_REQUEST_H
#define _MOS_REQUEST_H
#ifdef CONFIG_MOSIX

struct prequest
{
	struct prequest *rnext;		/* next in request chain */
	struct prequest *cnext;		/* caller chain */
	char rflags;			/* coordination flags - see below */
	char command;			/* ... */
	int param;			/* optional parameter */
	int param2;			/* optional parameter */
	caddr_t ereply;		 	/* extended reply address */
	int len;			/* optional extended reply length */
	int reply;			/* reply, if only one doubleword */
	struct wait_queue *waitq;	/* who is waiting for the reply */
};

/* flags: */

#define PR_WAITING	1       /* caller still waiting for the request */
#define PR_DONE	 	2	/* request complete */
#define PR_ERROR	4	/* request failed, reply contains errno */

/* commands: */

#define	PR_PROCFS_TASK_MEM		1
#define	PR_PROCFS_GET_ENV		2
#define	PR_PROCFS_GET_ARG		3
#define	PR_PROCFS_GET_STAT_PARTS	4
#define	PR_PROCFS_GET_STATM		5
#define	PR_PROCFS_MAP_INFO		6
#define	PR_PROCFS_MEM_READ		7
#define	PR_PROCFS_UPDATE_TIMES		8
#define	PR_PROCFS_GET_EXE		9
#define	PR_PTRACE_READ_LONG		10
#define	PR_PTRACE_WRITE_LONG		11
#define	PR_PTRACE_GET_STACK_LONG	12
#define	PR_PTRACE_PUT_STACK_LONG	13
#define	PR_PTRACE_GETREGS		14
#define	PR_PTRACE_SETREGS		15
#define	PR_PTRACE_PEEKUSER		16
#define	PR_PTRACE_POKEUSER		17
#define	PR_PTRACE_CONT			18
#define	PR_PTRACE_SINGLE_STEP		19
#define	PR_PTRACE_GETFPREGS		20
#define	PR_PTRACE_SETFPREGS		21

#ifdef CONFIG_MOSIX_DFSA
#define	PR_DFSA_SYNCHRONIZE		22
#endif CONFIG_MOSIX_DFSA

#define	SHOULD_ASK_PROCESS(p)	(p && (p->mosix.dflags & DDEPUTY) && (p != current || !(p->mosix.dflags & DINSCHED)))

extern int request_process_arg2(struct task_struct *, void *, int, int, int);
#define request_process(_t, _b, _c, _a) request_process_arg2(_t, _b, _c, _a, 0)
extern char *request_process_to_buf_arg(struct task_struct *, char *, int, int);
#define request_process_to_buf(_p,_b,_c) request_process_to_buf_arg(_p,_b,_c,0)

extern char *fill_common_ps_info(int *);
extern void pick_ps(struct task_struct *, struct prequest *);
extern void store_common_ps_info(void);
extern void stop_storing_common_ps_info(void);
extern void process_requests(void);
extern void discard_requests(void);
extern void process_only_easy_requests(void);
extern void request_on_remote(struct prequest *);

struct proc_remote_stat_parts
{
	unsigned long vsize, eip, esp;
	long priority;
	unsigned long rss, start_code, end_code, start_stack;
	long processor;
};

/* routines in other parts of the kernel that are called by MOSIX: */

extern int get_env(int, char *);
extern int get_arg(int, char *);
extern char *task_mem(struct task_struct *, char *);
extern void proc_get_stat_parts(struct proc_remote_stat_parts *, struct task_struct *);
extern int get_statm(int, char *);
extern int produce_own_map_info_line(char *, int);
extern int proc_read_my_mem(char *, caddr_t, int);
extern int proc_expound_dentry(char *);
extern int read_long(struct task_struct *, unsigned long, unsigned long *);
extern int write_long(struct task_struct *, unsigned long, unsigned long);
extern int get_stack_long(struct task_struct *, int);
extern int put_stack_long(struct task_struct *, int, unsigned long);
extern void ptrace_putregs(unsigned long *);
extern void ptrace_getregs(unsigned long *);
extern unsigned long ptrace_peekuser(long);
extern void ptrace_pokeuser(long, long);
extern void ptrace_cont(int);
extern void ptrace_single_step(void);
extern void ptrace_getfpregs(struct user_i387_struct *);
extern void ptrace_setfpregs(struct user_i387_struct *);
extern struct file *first_executable(void); 

#endif CONFIG_MOSIX
#endif
