/*
 * Copyright (c)  1998 - 2000, Amnon Barak, The Hebrew University of Jerusalem.
 * All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted
 * under the terms of version 2 or any later version of the GNU General
 * Public License, as published by the Free Software Foundation.
 *
 * THE HEBREW UNIVERSITY OF JERUSALEM  PROVIDES THIS SOFTWARE IN ITS "AS IS"
 * CONDITION. THE HEBREW UNIVERSITY OF JERUSALEM DISCLAIMS  ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 */
/*
 * Author(s): Amnon Shiloh, Oren Laadan.
 */
#ifndef _MOS_ROUTINES_H
#define _MOS_ROUTINES_H
#ifdef CONFIG_MOSIX

#include <mos/mosroutines.h>
#include <linux/stddef.h>

/*
 * a collection of routines in mos/?*.c that are neither:
 * 1) used by the rest of the kernel	 --> include/linux/mosix.h
 * 2) used by the MOSIX module		 --> include/mos/mosroutines.h
 * 3) in the MOSIX module		 --> include/mos/mosixroutines.h
 * 4) in the communication package	 --> include/mos/comm.h
 * 5) part of the DEPUTY/REMOTE protocol --> include/mos/protocol.h
 * 6) related to prequests		 --> include/mos/request.h
 * 7) related to DFSA			 --> include/mos/dfsa.h
 */

struct task_struct;
struct mosixnet;

/* task management: */

extern int remote_need_while_asleep(struct task_struct *);
extern int other_needs_while_asleep(void);
extern void remote_run_while_asleep(void);
extern void deputy_run_while_asleep(void);
extern int remote_pre_usermode_actions(void);
extern int local_pre_usermode_actions(void);
extern void per_task_ack_mosix_state(void);
extern void do_count_and_touch_mem(void);

/* migration: */

struct mig_request_h;
extern int mosix_mig_daemon(void *);
extern void mosix_do_add_to_whereto(struct mosix_task *, int);
extern int mosix_do_send_back_home(struct task_struct *);
extern int mosix_do_go_home(int);
extern void follow_whereto(void);
extern int mig_migrate(int);
extern void mig_set_lock(int);
extern int check_version(long);
extern void kickstart(void);
extern int run_over_dirty_pages(int (*func)(unsigned long, int), int);
extern void deputy_migrated(void);
extern int mig_send_request(int, int);
extern int mig_recv_request(struct mig_request_h **);
extern int mig_do_send(void);
extern int mig_do_receive(void);

/* deputy: */

extern void deputy_main_loop(void);
extern void undeputy(struct mosix_task *);
extern void deputy_async_requests(void);
extern void mosix_clear_all_remote_files(struct mosix_task *);
extern int fork_mosix_remote_files(struct task_struct *);
extern void coordinate(int, int);
extern int call_with_regs(void *, struct pt_regs *, struct pt_regs *);
extern int mostask_maps_ip(struct mosix_task *, struct inode *);
extern void deputy_communication_failed(int);
extern void deputy_die_on_communication(void) ATTRIB_NORET;

/* remote: */

extern void wait_for_permission_to_continue(void);
extern int remote_standard_system_call(int, struct pt_regs *);
extern struct file *get_remote_file(int, struct file *, unsigned long long, off_t, nopage_t);

/* data cache: */

struct syscall_h;
struct syscall_ret_h;
extern int copy_from_cache(char *, unsigned long, int, int *);
extern int copy_to_cache(unsigned long, char *, int, int *);
extern int zero_cache(char *, int, int *);
extern int strlen_cache(char *, int *);
extern int all_in_cache(unsigned long, unsigned int, int);
extern int any_in_cache(unsigned long, unsigned int);
extern int alloc_dcache(void);
extern void free_dcache(void);
extern void flush_dcache(void);
extern void flush_read_cache(void);
extern int dcache_ok(unsigned long, unsigned long, int);
extern int remote_unpack_read_cache_data(struct syscall_ret_h *);
extern void set_read_region(unsigned long, unsigned int);
extern void set_write_region(unsigned long, unsigned int);
extern char *construct_dcache_envelope(int *, int *, struct syscall_h *, char **);
extern int open_dcache_envelope(struct syscall_h *);
extern char *deputy_pack_read_cache_data(int *, struct syscall_ret_h *, char **);

/* configuration: */

extern int count_mosix_nodes(void); 
extern int mos_to_ascii(int, char *, int);
extern int mosix_config_get_table(struct mosixnet **, int, int);
extern int mosix_config_set_table(struct mosixnet *, int, int);
extern int mosix_config_get_pe(void);
extern int mosix_config_set_pe(int);
extern int mosix_config_get_tentative_pe(void);
extern int mosix_config_get_limit(void);
extern int count_mosix_nodes(void);
extern int config_get_status(int);
extern int config_set_status(int);
extern int i_am_in_a_wrong_place(void);
extern void done_checking_conf(void);

/* decay management: */

extern int decay_inherit(int);
extern int decay_exec(int);
extern int decay_execonce(int);
extern int decay_set(int, int, int);
extern int decay_get(int);
extern void decay_clear(void);
extern void deputy_update_remote_decay(void);

/* MOSIX administration: */

extern int admin_get_mode(int);
extern int admin_set_mode(int, int);
extern int cant_turn_off_admin(int);
extern int expel(int);
extern int bring(void);
extern void proc_update_costs(void);

/* debugging: */

extern void init_mosix_log(struct mosix_task *);
extern void clear_mosix_log(struct mosix_task *);

/* macros: */

#define	MOSIX_TO_TASK(m) ((struct task_struct *)(((char *)m)-offsetof(struct task_struct,mosix)))

#ifdef CONFIG_MOSIX_DFSA
#define	URGENT_REMOTE_CONDITIONS(m)	((m)->whereto || \
						 ((m)->dflags & DSYNCDFSASYS))
#else
#define	URGENT_REMOTE_CONDITIONS(m)	((m)->whereto)
#endif CONFIG_MOSIX_DFSA
#endif CONFIG_MOSIX

#endif
