/*
 * Copyright (c)  1998 - 2000, Amnon Barak, The Hebrew University of Jerusalem.
 * All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted
 * under the terms of version 2 or any later version of the GNU General
 * Public License, as published by the Free Software Foundation.
 *
 * THE HEBREW UNIVERSITY OF JERUSALEM  PROVIDES THIS SOFTWARE IN ITS "AS IS"
 * CONDITION. THE HEBREW UNIVERSITY OF JERUSALEM DISCLAIMS  ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 */
/*
 * Author(s): Amnon Shiloh, Oren Laadan.
 */

/* THIS IS AN OPERATING-SYSTEM INDEPENDENT FILE:		*/
/* DO NOT MENTION HERE ANY STRUCTURES, VARIABLES OR DEFINITIONS	*/
/* FROM THE PARTICULAR HOSTING OPERATING-SYSTEM KERNEL		*/

#ifndef _MOS_VERSION_H
#define _MOS_VERSION_H

/* The following is the EXTERNAL MOSIX version, used to enforce compatibility
 * between different MOSIX nodes when migrating - as opposed to the later
 * INTERNAL version, being used to match between a kernel and its MOSIX module:
 */
#define MOSIX_MIGRATION_VERSION	0x000010L

/* The following version-numbers make sure that no MOSIX module
 * is allowed to load onto an too-early kernel and vice-versa.
 * POLICIES:
 * 1) major changes should modify the 10K's of both kernel and module versions
 *    and set them to both be the same.
 * 2) moderate changes should modify the 100's of both kernel and module
 *    versions and set them to both be the same.
 * 3) minor changes in either kernel or module affect only the last 2 digits
 *    of the kernel/module version respectively.
 *
 * 4) When making a change in the kernel that makes it incompatible with
 *    older modules, update MIN_MODULE_VERSION.
 * 5) When making a change in the module that makes it incompatible with
 *    older kernels, update MIN_KERNEL_VERSION.
 */
#define MOSIX_KERNEL_VERSION	 9707	/* version 0.97.7 */
#define	MOSIX_MODULE_VERSION	 9707	/* version 0.97.7 */
#define	MIN_MODULE_VERSION	 9707	/* that is compatible with kernel */
#define	MIN_KERNEL_VERSION	 9707	/* that is compatible with module */

#define MOSIX_VERSION_PARTS(x) \
        (x)/10000, ((x)%10000)/100, (x)%100
	
#if MIN_MODULE_VERSION > MOSIX_MODULE_VERSION
#error Oops, forgot to update MOSIX_MODULE_VERSION!
#endif
#if MIN_KERNEL_VERSION > MOSIX_KERNEL_VERSION
#error Oops, forgot to update MOSIX_KERNEL_VERSION!
#endif

extern int mosix_kernel_version;
extern int min_moskernel_version;
extern int mosix_module_version;
extern int min_mosmodule_version;
extern int mosmodule_compile;

#endif _MOS_VERSION_H
