/*
 * Copyright (c)  1998 - 2000, Amnon Barak, The Hebrew University of Jerusalem.
 * All rights reserved.
 *
 * Permission to use, copy and distribute this software is hereby granted
 * under the terms of version 2 or any later version of the GNU General
 * Public License, as published by the Free Software Foundation.
 *
 * THE HEBREW UNIVERSITY OF JERUSALEM  PROVIDES THIS SOFTWARE IN ITS "AS IS"
 * CONDITION. THE HEBREW UNIVERSITY OF JERUSALEM DISCLAIMS  ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 */
/*
 * Author(s): Amnon Shiloh, Oren Laadan.
 */

/* THIS IS AN OPERATING-SYSTEM INDEPENDENT FILE:		*/
/* DO NOT MENTION HERE ANY STRUCTURES, VARIABLES OR DEFINITIONS	*/
/* FROM THE PARTICULAR HOSTING OPERATING-SYSTEM KERNEL		*/

#ifndef _MOSIX_BALANCE_H
#define _MOSIX_BALANCE_H

#ifdef __KERNEL__

#define MOSIX_BALANCE_VERSION	0x000020L

#define	MAXKNOWNUP	45
#if defined(CONFIG_MOSIX_LIMITED) && CONFIG_MOSIX_NODES <= 45
#undef MAXKNOWNUP
#define	MAXKNOWNUP	(CONFIG_MOSIX_NODES-1)
#endif

struct loadinfo {
	unsigned short pe;
	unsigned short speed;	/* in SMP: of each CPU (not the total) */
	unsigned long load;
	unsigned short ncpus;
	unsigned short util;	/* in SMP: max=MF*smp_num_cpus */
	unsigned short status;
	unsigned short free_slots;
	unsigned int mem;
	unsigned int rmem;
	unsigned int tmem;
#ifdef CONFIG_MOSIX_TOPOLOGY
	struct opcost costs[MAX_MOSIX_TOPOLOGY];
	struct mfs_cost mfscosts[MAX_MOSIX_TOPOLOGY];
#endif CONFIG_MOSIX_TOPOLOGY
};

struct miginfo {
	int aload;
	unsigned long pages;
};

struct infomsg {
	long version;
	int topology;
	int serialno;
	struct loadinfo load;
};

#define SIZEOF_INFOMSG	sizeof(struct infomsg)

struct aload_h
{
	int mosix_is_loaded;
	int aload;
	int freepages;
#ifdef CONFIG_MOSIX_TOPOLOGY
	struct mfs_cost mfscosts[MAX_MOSIX_TOPOLOGY];
#endif CONFIG_MOSIX_TOPOLOGY
};

#define INFO_WIN	8
#define	BALANCING_TIMEOUT	10

/* following is a multiple to be used for statistical values - primarily the
 * number of system-calls.  Statistical values are counted in chunks of
 * EVENT_COUNTER_VALUE, rather than simply 1, for the following two reasons:
 * 1) to prevent being lost by decaying
 * 2) making the contribution of a system-call to "iocounter" reflect its
 *    impact, which is considerably more relative to 1-byte of I/O:
 *    "iocounter" need not be accurate since it is only used to trigger
 *    considerations for IOBALANCE migrations, but it is good to maintain
 *    an order of magnitude to prevent multiplications in "iocounter"
 *    computations.
 */
#define	EVENT_COUNTER_VALUE	1024
#define	EVENT_COUNTER_SHIFT	10

/* MDP constants: */
#define	KEEP_FREE_PORTION	16	/* 1/16 of memory */
#define	MAX_PAGES_TO_KEEP_FREE	(0x600000/page_size)	/* but no more than 6MB */
#define	MIN_EXPECTED_PROC_SIZE	(0x80000/page_size)	/* when freeing memory, at least this */
#define	MIN_MCHOOSE_AGAIN	1000000	/* youngest (RT) to mconsider */
#define	MAX_MCHOOSE_AGAIN	4000000	/* no elder-advantage beyond this */
#define	UTIL_TOLLERANCE		(8/10)	/* under this, sure trashing */

extern struct loadinfo loadinfo[INFO_WIN];

extern int export_load;		/* load reported to other processors */
extern int stable_export;	/* machine dependent stabilizing factor */
extern unsigned acpuse;		/* accumulated cpu utilization */
extern unsigned coming_in;	/* number of arriving processes */
extern unsigned came_lately4;	/* processes that arrived lately (*4) */
extern unsigned load_left;	/* load of processes that just left */
extern int standard_speed;	/* yardstick */
extern struct opcost deputy_here[MAX_MOSIX_TOPOLOGY]; /* DEPUTY I/O overheads here */

extern int pages_to_keep_free;	/* # of pages to attempt to keep free */
extern int latest_free_mem;	/* in pages */

#define	BUMP_HEAD do { register struct mosix_task *m = curmos(); \
			unsigned long oldcounter = m->iocounter

#define	BUMP_TAIL if((m->iocounter ^ oldcounter) & 0xfff00000) {\
			mosix_do_add_to_whereto(m, IOBALANCE); \
			m->iocounter = 0;} } while(0)

#define bump_statistics(s) BUMP_HEAD; \
	m->s += EVENT_COUNTER_VALUE; \
	m->iocounter += EVENT_COUNTER_VALUE; \
	BUMP_TAIL
#define	bump_statistic_amount(type,b) BUMP_HEAD; \
	m->n##type##s += COPY_COUNTER_MULTIPLIER; \
	m->type##bytes += b; \
	m->iocounter += COPY_COUNTER_MULTIPLIER + b; \
	BUMP_TAIL

extern void mosix_info_daemon(void); 
extern void load_balance(void);
extern void memory_balance(void);
extern void info_init(void);
extern void info_startup(void);
extern void info_reconfig(void);
extern void init_mdp(void);
extern void mosix_load_init(void);
extern void mosix_load_stop(void);
extern void raise_all(int, int);
int export_mem(void);
int memory_badly_required(void);
int memory_relief_quality(struct mosix_task *, int);

#define	DMILLION 1000000.0

#define	MAGIC_HZZ	0x78A3B9C2 /* do not change - a bit more than random */

#define	NULL	0

#endif __KERNEL__

#endif
